// jshint maxerr:1000
/*
 * Copyright (c) 2023, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* ===============================================================================================================
   ===============================================================================================================
                                                     BASIC TAB TOOL 
   ===============================================================================================================
   =============================================================================================================== */
let Basic_Error = {
    /**
     * An object holding all the error/condition checking functions. 
     * If the condition is triggered the function returns a list [1, 'error text'];
     * otherwise the function returns [0, ''] 
     */
    Rules: {
        check_radarCubeSize: function() {
            if (mmWaveBasic.Output.radar_cube_size > mmWaveBasic.Intermediate.maximum_radar_cube_size) {
                ['ti_widget_container_range_resolution',
                 'ti_widget_container_velocity_resolution',
                 'ti_widget_container_maximum_detectable_range',
                 'ti_widget_container_maximum_velocity'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1, 'The radar cube size is larger than the available memory on the device: Increase the "Range Resolution"/"Velocity Resolution", and/or decrease the "Maximum Range"/"Maximum Velocity'];
            }
            return [0,''];
        },
        check_periodicity_upper: function() {
            if (mmWaveBasic.Intermediate.periodicity < mmWaveBasic.Output.active_frame_time) {
                ['ti_widget_container_maximum_velocity',
                 'ti_widget_container_velocity_resolution',
                 'ti_widget_container_measurement_rate'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1, 'Frame rate is too high and causes higher than 100% duty cycle: Decrease the "Meaurement Rate", decrease the "Maximum Velocity", and/or increase the "Velocity Resolution"']
            }
            return [0,''];
        },
        check_periodicity_lower: function() {
            if (mmWaveBasic.Output.active_frame_time < 0.2) {
                ['ti_widget_container_maximum_velocity',
                 'ti_widget_container_velocity_resolution',
                 'ti_widget_container_measurement_rate'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1,'Active Frame Time is too slow, it cannot be less that 0.2 ms: Decrease the "Frame Rate", decrease the "Maximum Velocity", and/or increase the "Velocity Resolution'];
            }
            return [0,''];
        },
        check_rampSlope: function() {
            if (((mmWaveBasic.Input.device === 'xWR6843' || mmWaveBasic.Input.device === 'xWRL6432') && mmWaveBasic.Output.ramp_slope > 266) ||
                ((mmWaveBasic.Input.device !== 'xWR6843' && mmWaveBasic.Input.device !== 'xWRL6432')  && mmWaveBasic.Output.ramp_slope > 100)) 
            {
                ['ti_widget_container_range_resolution',
                 'ti_widget_container_maximum_velocity'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1,'Frequency slope of this chirp design is too high (>100MHz/us for 77GHz or >266MHz for 60GHz): Increase the "Range Resolution" and/or decrease the "Maximum Velocity"'];
            }
            else if (mmWaveBasic.Output.ramp_slope <= 0) {
                ['ti_widget_container_range_resolution',
                 'ti_widget_container_maximum_velocity'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1,'Frequency slope of this chirp design is negative (<0MHz/us): Increase the "Range Resolution" and/or decrease the "Maximum Velocity"'];
            }
            return [0,''];
        },
        check_chirpRepPeriod: function() {
            if (mmWaveBasic.Output.chirp_repetition_period > mmWaveBasic.Intermediate.max_chirp_repetition_period) {
                ['ti_widget_container_range_resolution',
                 'ti_widget_container_maximum_velocity'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                return [1,'Chirp time is too long: Increase the "Range Resolution", and/or decrease the "Maximum Velocity"'];
            }
            return [0,''];
        },
        check_samplingFrequency: function() {
            if (mmWaveBasic.Output.sampling_frequency > mmWaveBasic.Intermediate.maximum_sampling_frequency) {
                ['ti_widget_container_range_resolution',
                 'ti_widget_container_maximum_velocity',
                 'ti_widget_container_maximum_detectable_range'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                {
                }
                return [1, 'Sampling frequency is higher than the maximum sampling frequency (>' + mmWaveBasic.Intermediate.maximum_sampling_frequency + ' Msps): Decrease the "Maximum Velocity", decrease the "Maximum Detectable Range", and/or increase the "Range Resolution"'];
            }
            return [0,''];
        },
        check_maxBeatFreq: function() {
            if (mmWaveBasic.Output.maximum_beat_frequency > mmWaveBasic.Intermediate.maximum_sampling_frequency) {
                ['ti_widget_container_range_resolution',
                 'ti_widget_container_maximum_velocity',
                 'ti_widget_container_maximum_detectable_range'].forEach(widget => {
                    Util.Html.highlightPropWithPrecedence(templateObj.$[widget], 1, 'conditional');
                });
                {
                }
                return [1, 'Beat frequency is higher than the maximum I/F frequency (>' +  mmWaveBasic.Intermediate.maximum_IF_bandwidth + ' MHz): Decrease the "Maximum Velocity", decrease the "Maximum Detectable Range", and/or increase the "Range Resolution"'];
            }
            return [0,''];
        },
        check_totalSweepBandwidth: function() {
            if (mmWaveBasic.Output.total_sweep_bandwidth > mmWaveBasic.Output.maximum_bandwidth) {
                Util.Html.highlightPropWithPrecedence(templateObj.$.ti_widget_container_range_resolution, 1, 'conditional');
                return [1, 'The "Bandwidth" is larger than the "Maximum Bandwidth": Increase the "Range Resolution"'];
            }
            return [0,''];
        },
        check_maxRange_typicalDetected: function() {
            if ((mmWaveBasic.Input.device === 'xWRL6432' || mmWaveBasic.Input.device === 'xWRL1432') && mmWaveBasic.Output.max_range_for_typical_detectable_object > 40) {
                return [1, 'The maximum range is a theoretical calculation and, in reality, is not likely to be this high for this device due to limited transmit power.']
            }
            else if (mmWaveBasic.Output.max_range_for_typical_detectable_object > 200) {
                return [1, 'The maximum range is a theoretical calculation and, in reality, is not likely to be this high for this device due to limited transmit power.']
            }
            return [0,''];
        },
        check_maxRange: function() {
            if ((mmWaveBasic.Input.device === 'xWRL6432' || mmWaveBasic.Input.device === 'xWRL1432') && mmWaveBasic.Input.maximum_detectable_range > 40) {
                return [1, 'The maximum range for a typical detected object is a theoretical calculation and, in reality, is not likely to be this high for this device due to limited transmit power.']
            }
            else if (mmWaveBasic.Input.maximum_detectable_range > 200) {
                return [1, 'The maximum range is, in reality, is not likely to be this high for this device due to limited transmit power.']
            }
            return [0,''];
        },
        check_rangeResolution: function() {
            var theoreticalRangeRes_m = Basic_Globals.Constants.lightSpeed / (2 * mmWaveBasic.Output.maximum_bandwidth * 1e6);
            var theoreticalRangeRes_cm = theoreticalRangeRes_m * 1e2;
            if (mmWaveBasic.Input.range_resolution < theoreticalRangeRes_cm) {
                return [1, 'The desired range resolution cannot be lower than the theoretical minimum, calculated by c / (2 * Bandwidth) = ' + theoreticalRangeRes_cm + '.']
            }
            return [0,''];
        }
    },
};

let Basic_ErrorMaps = {
    /**
     * A mapping to tie the front end widgets to their error-checking function.
     */
    Err: [
        [['ti_widget_container_radar_cube_size'],           Basic_Error.Rules.check_radarCubeSize],
        [['ti_widget_container_periodicity'],               Basic_Error.Rules.check_periodicity_upper],
        [['ti_widget_container_periodicity'],               Basic_Error.Rules.check_periodicity_lower],
        [['ti_widget_container_ramp_slope'],                Basic_Error.Rules.check_rampSlope],
        [['ti_widget_container_chirp_repetition_period'],   Basic_Error.Rules.check_chirpRepPeriod],
        [['ti_widget_container_sampling_frequency'],        Basic_Error.Rules.check_samplingFrequency],
        [['ti_widget_container_maximum_beat_frequency'],    Basic_Error.Rules.check_maxBeatFreq],
        [['ti_widget_container_total_sweep_bandwidth'],     Basic_Error.Rules.check_totalSweepBandwidth],
        [['ti_widget_container_range_resolution'],          Basic_Error.Rules.check_rangeResolution]
        
    ],
    Warn: [
        [['ti_widget_container_max_range_for_det_obj'],     Basic_Error.Rules.check_maxRange_typicalDetected],
        [['ti_widget_container_maximum_detectable_range'],  Basic_Error.Rules.check_maxRange]
    ],
    /**
     * Cells that are ever highlighted as "related" (conditional)
     */
    Related: [
        'ti_widget_container_range_resolution',
        'ti_widget_container_velocity_resolution',
        'ti_widget_container_maximum_detectable_range',
        'ti_widget_container_maximum_velocity',
        'ti_widget_container_measurement_rate',
    ],
};